import os
import pathlib
import uuid

REPOSITORY_ROOT = pathlib.Path(__file__).parent.parent.parent


def generate(category, example_name, c_source_file):
    guid = str(uuid.uuid4()).upper()
    text = f"""
<?xml version="1.0" encoding="utf-8"?>
<Project DefaultTargets="Build" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
  <PropertyGroup Label="Globals">
    <ProjectGuid>{{{guid}}}</ProjectGuid>
  </PropertyGroup>
  <Import Project="$(VCTargetsPath)\\Microsoft.Cpp.Default.props" />
  <Import Project="$(VCTargetsPath)\\Microsoft.Cpp.props" />
  <ItemGroup>
    <None Include="$(SolutionDir)\\..\\examples\\{category}\\{example_name}\\README.txt" />
    <ClCompile Include="$(SolutionDir)\\..\\examples\\{category}\\{example_name}\\{c_source_file}" />
  </ItemGroup>
  <Import Project="$(VCTargetsPath)\\Microsoft.Cpp.targets" />
</Project>
""".strip()

    project_file = REPOSITORY_ROOT / "VisualC" / "examples" / category / example_name / f"{example_name}.vcxproj"

    if project_file.exists():
        print("Skipping:", project_file)
        return

    print("Generating file:", project_file)
    os.makedirs(project_file.parent, exist_ok=True)
    with open(project_file, "w", encoding="utf-8") as f:
        f.write(text)


def get_c_source_filename(example_dir: pathlib.Path):
    """Gets the one and only C source file name in the directory of the example."""
    c_files = [f.name for f in example_dir.iterdir() if f.name.endswith(".c")]
    assert len(c_files) == 1
    return c_files[0]


def main():
    path = REPOSITORY_ROOT / "examples"
    for category in path.iterdir():
        if category.is_dir():
            for example in category.iterdir():
                generate(category.name, example.name, get_c_source_filename(example))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import argparse
import contextlib
import logging
import os
import pathlib
import shlex
import sys
import time
from typing import Optional
import urllib.parse

from selenium import webdriver
import selenium.common.exceptions
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait


logger = logging.getLogger(__name__)


class SDLSeleniumTestDriver:
    def __init__(self, server: str, test: str, arguments: list[str], browser: str, firefox_binary: Optional[str]=None, chrome_binary: Optional[str]=None):
        self. server = server
        self.test = test
        self.arguments = arguments
        self.chrome_binary = chrome_binary
        self.firefox_binary = firefox_binary
        self.driver = None
        self.stdout_printed = False
        self.failed_messages: list[str] = []
        self.return_code = None

        options = [
            "--headless",
        ]

        driver_contructor = None
        match browser:
            case "firefox":
                driver_contructor = webdriver.Firefox
                driver_options = webdriver.FirefoxOptions()
                if self.firefox_binary:
                    driver_options.binary_location = self.firefox_binary
            case "chrome":
                driver_contructor = webdriver.Chrome
                driver_options = webdriver.ChromeOptions()
                if self.chrome_binary:
                    driver_options.binary_location = self.chrome_binary
                options.append("--no-sandbox")
        if driver_contructor is None:
            raise ValueError(f"Invalid {browser=}")
        for o in options:
            driver_options.add_argument(o)
        logger.debug("About to create driver")
        self.driver = driver_contructor(options=driver_options)

    @property
    def finished(self):
        return len(self.failed_messages) > 0 or self.return_code is not None

    def __del__(self):
        if self.driver:
            self.driver.quit()

    @property
    def url(self):
        req = {
            "loghtml": "1",
            "SDL_ASSERT": "abort",
        }
        for key, value in os.environ.items():
            if key.startswith("SDL_"):
                req[key] = value
        req.update({f"arg_{i}": a for i, a in enumerate(self.arguments, 1) })
        req_str = urllib.parse.urlencode(req)
        return f"{self.server}/{self.test}.html?{req_str}"

    @contextlib.contextmanager
    def _selenium_catcher(self):
        try:
            yield
            success = True
        except selenium.common.exceptions.UnexpectedAlertPresentException as e:
            # FIXME: switch context, verify text of dialog and answer "a" for abort
            wait = WebDriverWait(self.driver, timeout=2)
            try:
                alert = wait.until(lambda d: d.switch_to.alert)
            except selenium.common.exceptions.NoAlertPresentException:
                self.failed_messages.append(e.msg)
                return False
            self.failed_messages.append(alert)
            if "Assertion failure" in e.msg and "[ariA]" in e.msg:
                alert.send_keys("a")
                alert.accept()
            else:
                self.failed_messages.append(e.msg)
            success = False
        return success

    def get_stdout_and_print(self):
        if self.stdout_printed:
            return
        with self._selenium_catcher():
            div_terminal = self.driver.find_element(by=By.ID, value="terminal")
            assert div_terminal
            text = div_terminal.text
            print(text)
            self.stdout_printed = True

    def update_return_code(self):
        with self._selenium_catcher():
            div_process_quit = self.driver.find_element(by=By.ID, value="process-quit")
            if not div_process_quit:
                return
            if div_process_quit.text != "":
                try:
                    self.return_code = int(div_process_quit.text)
                except ValueError:
                    raise ValueError(f"process-quit element contains invalid data: {div_process_quit.text:r}")

    def loop(self):
        print(f"Connecting to \"{self.url}\"", file=sys.stderr)
        self.driver.get(url=self.url)
        self.driver.implicitly_wait(0.2)

        while True:
            self.update_return_code()
            if self.finished:
                break
            time.sleep(0.1)

        self.get_stdout_and_print()
        if not self.stdout_printed:
            self.failed_messages.append("Failed to get stdout/stderr")



def main() -> int:
    parser = argparse.ArgumentParser(allow_abbrev=False, description="Selenium SDL test driver")
    parser.add_argument("--browser", default="firefox", choices=["firefox", "chrome"], help="browser")
    parser.add_argument("--server", default="http://localhost:8080", help="Server where SDL tests live")
    parser.add_argument("--verbose", action="store_true", help="Verbose logging")
    parser.add_argument("--chrome-binary", help="Chrome binary")
    parser.add_argument("--firefox-binary", help="Firefox binary")

    index_double_dash = sys.argv.index("--")
    if index_double_dash < 0:
        parser.error("Missing test arguments. Need -- <FILENAME> <ARGUMENTS>")
    driver_arguments = sys.argv[1:index_double_dash]
    test = pathlib.Path(sys.argv[index_double_dash+1]).name
    test_arguments = sys.argv[index_double_dash+2:]

    args = parser.parse_args(args=driver_arguments)

    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.INFO)

    logger.debug("driver_arguments=%r test=%r test_arguments=%r", driver_arguments, test, test_arguments)

    sdl_test_driver = SDLSeleniumTestDriver(
        server=args.server,
        test=test,
        arguments=test_arguments,
        browser=args.browser,
        chrome_binary=args.chrome_binary,
        firefox_binary=args.firefox_binary,
    )
    sdl_test_driver.loop()

    rc = sdl_test_driver.return_code
    if sdl_test_driver.failed_messages:
        for msg in sdl_test_driver.failed_messages:
            print(f"FAILURE MESSAGE: {msg}", file=sys.stderr)
        if rc == 0:
            print(f"Test signaled success (rc=0) but a failure happened", file=sys.stderr)
            rc = 1
    sys.stdout.flush()
    logger.info("Exit code = %d", rc)
    return rc


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
#
# This script renames SDL macros in the specified paths

import argparse
import pathlib
import re


class TextReplacer:
    def __init__(self, macros, repl_format):
        if isinstance(macros, dict):
            macros_keys = macros.keys()
        else:
            macros_keys = macros
        self.macros = macros
        self.re_macros = re.compile(r"\W(" + "|".join(macros_keys) + r")(?:\W|$)")
        self.repl_format = repl_format

    def apply(self, contents):
        def cb(m):
            macro = m.group(1)
            original = m.group(0)
            match_start, _ = m.span(0)
            platform_start, platform_end = m.span(1)
            if isinstance(self.macros, dict):
                repl_args = (macro, self.macros[macro])
            else:
                repl_args = macro,
            new_text = self.repl_format.format(*repl_args)
            r = original[:(platform_start-match_start)] + new_text + original[platform_end-match_start:]
            return r
        contents, _ = self.re_macros.subn(cb, contents)

        return contents


class MacrosCheck:
    def __init__(self):
        self.renamed_platform_macros = TextReplacer(RENAMED_MACROS, "{1}")
        self.deprecated_platform_macros = TextReplacer(DEPRECATED_PLATFORM_MACROS, "{0} /* {0} has been removed in SDL3 */")

    def run(self, contents):
        contents = self.renamed_platform_macros.apply(contents)
        contents = self.deprecated_platform_macros.apply(contents)
        return contents


def apply_checks(paths):
    checks = (
        MacrosCheck(),
    )

    for entry in paths:
        path = pathlib.Path(entry)
        if not path.exists():
            print("{} does not exist, skipping".format(entry))
            continue
        apply_checks_in_path(path, checks)


def apply_checks_in_file(file, checks):
    try:
        with file.open("r", encoding="UTF-8", newline="") as rfp:
            original = rfp.read()
            contents = original
            for check in checks:
                contents = check.run(contents)
            if contents != original:
                with file.open("w", encoding="UTF-8", newline="") as wfp:
                    wfp.write(contents)
    except UnicodeDecodeError:
        print("%s is not text, skipping" % file)
    except Exception as err:
        print("%s" % err)


def apply_checks_in_dir(path, checks):
    for entry in path.glob("*"):
        if entry.is_dir():
            apply_checks_in_dir(entry, checks)
        else:
            print("Processing %s" % entry)
            apply_checks_in_file(entry, checks)


def apply_checks_in_path(path, checks):
        if path.is_dir():
            apply_checks_in_dir(path, checks)
        else:
            apply_checks_in_file(path, checks)


def main():
    parser = argparse.ArgumentParser(fromfile_prefix_chars='@', description="Rename macros for SDL3")
    parser.add_argument("args", nargs="*", help="Input source files")
    args = parser.parse_args()

    try:
        apply_checks(args.args)
    except Exception as e:
        print(e)
        return 1


RENAMED_MACROS = {
    "__AIX__": "SDL_PLATFORM_AIX",
    "__HAIKU__": "SDL_PLATFORM_HAIKU",
    "__BSDI__": "SDL_PLATFORM_BSDI",
    "__FREEBSD__": "SDL_PLATFORM_FREEBSD",
    "__HPUX__": "SDL_PLATFORM_HPUX",
    "__IRIX__": "SDL_PLATFORM_IRIX",
    "__LINUX__": "SDL_PLATFORM_LINUX",
    "__OS2__": "SDL_PLATFORM_OS2",
    # "__ANDROID__": "SDL_PLATFORM_ANDROID,
    "__APPLE__": "SDL_PLATFORM_APPLE",
    "__TVOS__": "SDL_PLATFORM_TVOS",
    "__IPHONEOS__": "SDL_PLATFORM_IOS",
    "__MACOSX__": "SDL_PLATFORM_MACOS",
    "__NETBSD__": "SDL_PLATFORM_NETBSD",
    "__OPENBSD__": "SDL_PLATFORM_OPENBSD",
    "__OSF__": "SDL_PLATFORM_OSF",
    "__QNXNTO__": "SDL_PLATFORM_QNXNTO",
    "__RISCOS__": "SDL_PLATFORM_RISCOS",
    "__SOLARIS__": "SDL_PLATFORM_SOLARIS",
    "__PSP__": "SDL_PLATFORM_PSP",
    "__PS2__": "SDL_PLATFORM_PS2",
    "__VITA__": "SDL_PLATFORM_VITA",
    "__3DS__": "SDL_PLATFORM_3DS",
    # "__unix__": "SDL_PLATFORM_UNIX,
    "__XBOXSERIES__": "SDL_PLATFORM_XBOXSERIES",
    "__XBOXONE__": "SDL_PLATFORM_XBOXONE",
    "__WINDOWS__": "SDL_PLATFORM_WINDOWS",
    "__WIN32__": "SDL_PLATFORM_WIN32",
    # "__CYGWIN_": "SDL_PLATFORM_CYGWIN",
    "__WINGDK__": "SDL_PLATFORM_WINGDK",
    "__GDK__": "SDL_PLATFORM_GDK",
    # "__EMSCRIPTEN__": "SDL_PLATFORM_EMSCRIPTEN",
}

DEPRECATED_PLATFORM_MACROS = {
    "__DREAMCAST__",
    "__NACL__",
    "__PNACL__",
    "__WINDOWS__",
    "__WINRT__",
    "SDL_ALTIVEC_BLITTERS",
    "SDL_ARM_NEON_BLITTERS",
    "SDL_ARM_SIMD_BLITTERS",
    "SDL_ATOMIC_DISABLED",
    "SDL_AUDIO_DISABLED",
    "SDL_AUDIO_DRIVER_AAUDIO",
    "SDL_AUDIO_DRIVER_ALSA",
    "SDL_AUDIO_DRIVER_ALSA_DYNAMIC",
    "SDL_AUDIO_DRIVER_ANDROID",
    "SDL_AUDIO_DRIVER_ARTS",
    "SDL_AUDIO_DRIVER_ARTS_DYNAMIC",
    "SDL_AUDIO_DRIVER_COREAUDIO",
    "SDL_AUDIO_DRIVER_DISK",
    "SDL_AUDIO_DRIVER_DSOUND",
    "SDL_AUDIO_DRIVER_DUMMY",
    "SDL_AUDIO_DRIVER_EMSCRIPTEN",
    "SDL_AUDIO_DRIVER_ESD",
    "SDL_AUDIO_DRIVER_ESD_DYNAMIC",
    "SDL_AUDIO_DRIVER_FUSIONSOUND",
    "SDL_AUDIO_DRIVER_FUSIONSOUND_DYNAMIC",
    "SDL_AUDIO_DRIVER_HAIKU",
    "SDL_AUDIO_DRIVER_JACK",
    "SDL_AUDIO_DRIVER_JACK_DYNAMIC",
    "SDL_AUDIO_DRIVER_N3DS",
    "SDL_AUDIO_DRIVER_NAS",
    "SDL_AUDIO_DRIVER_NAS_DYNAMIC",
    "SDL_AUDIO_DRIVER_NETBSD",
    "SDL_AUDIO_DRIVER_OPENSLES",
    "SDL_AUDIO_DRIVER_OS2",
    "SDL_AUDIO_DRIVER_OSS",
    "SDL_AUDIO_DRIVER_PAUDIO",
    "SDL_AUDIO_DRIVER_PIPEWIRE",
    "SDL_AUDIO_DRIVER_PIPEWIRE_DYNAMIC",
    "SDL_AUDIO_DRIVER_PS2",
    "SDL_AUDIO_DRIVER_PSP",
    "SDL_AUDIO_DRIVER_PULSEAUDIO",
    "SDL_AUDIO_DRIVER_PULSEAUDIO_DYNAMIC",
    "SDL_AUDIO_DRIVER_QSA",
    "SDL_AUDIO_DRIVER_SNDIO",
    "SDL_AUDIO_DRIVER_SNDIO_DYNAMIC",
    "SDL_AUDIO_DRIVER_SUNAUDIO",
    "SDL_AUDIO_DRIVER_VITA",
    "SDL_AUDIO_DRIVER_WASAPI",
    "SDL_AUDIO_DRIVER_WINMM",
    "SDL_CPUINFO_DISABLED",
    "SDL_DEFAULT_ASSERT_LEVEL",
    "SDL_EVENTS_DISABLED",
    "SDL_FILESYSTEM_ANDROID",
    "SDL_FILESYSTEM_COCOA",
    "SDL_FILESYSTEM_DISABLED",
    "SDL_FILESYSTEM_DUMMY",
    "SDL_FILESYSTEM_EMSCRIPTEN",
    "SDL_FILESYSTEM_HAIKU",
    "SDL_FILESYSTEM_N3DS",
    "SDL_FILESYSTEM_OS2",
    "SDL_FILESYSTEM_PS2",
    "SDL_FILESYSTEM_PSP",
    "SDL_FILESYSTEM_RISCOS",
    "SDL_FILESYSTEM_UNIX",
    "SDL_FILESYSTEM_VITA",
    "SDL_FILESYSTEM_WINDOWS",
    "SDL_FILE_DISABLED",
    "SDL_HAPTIC_ANDROID",
    "SDL_HAPTIC_DINPUT",
    "SDL_HAPTIC_DISABLED",
    "SDL_HAPTIC_DUMMY",
    "SDL_HAPTIC_IOKIT",
    "SDL_HAPTIC_LINUX",
    "SDL_HAPTIC_XINPUT",
    "SDL_HAVE_LIBDECOR_GET_MIN_MAX",
    "SDL_HAVE_MACHINE_JOYSTICK_H",
    "SDL_HIDAPI_DISABLED",
    "SDL_INPUT_FBSDKBIO",
    "SDL_INPUT_LINUXEV",
    "SDL_INPUT_LINUXKD",
    "SDL_INPUT_WSCONS",
    "SDL_IPHONE_KEYBOARD",
    "SDL_IPHONE_LAUNCHSCREEN",
    "SDL_JOYSTICK_ANDROID",
    "SDL_JOYSTICK_DINPUT",
    "SDL_JOYSTICK_DISABLED",
    "SDL_JOYSTICK_DUMMY",
    "SDL_JOYSTICK_EMSCRIPTEN",
    "SDL_JOYSTICK_HAIKU",
    "SDL_JOYSTICK_HIDAPI",
    "SDL_JOYSTICK_IOKIT",
    "SDL_JOYSTICK_LINUX",
    "SDL_JOYSTICK_MFI",
    "SDL_JOYSTICK_N3DS",
    "SDL_JOYSTICK_OS2",
    "SDL_JOYSTICK_PS2",
    "SDL_JOYSTICK_PSP",
    "SDL_JOYSTICK_RAWINPUT",
    "SDL_JOYSTICK_USBHID",
    "SDL_JOYSTICK_VIRTUAL",
    "SDL_JOYSTICK_VITA",
    "SDL_JOYSTICK_WGI",
    "SDL_JOYSTICK_XINPUT",
    "SDL_LIBSAMPLERATE_DYNAMIC",
    "SDL_LIBUSB_DYNAMIC",
    "SDL_LOADSO_DISABLED",
    "SDL_LOADSO_DLOPEN",
    "SDL_LOADSO_DUMMY",
    "SDL_LOADSO_LDG",
    "SDL_LOADSO_OS2",
    "SDL_LOADSO_WINDOWS",
    "SDL_LOCALE_DISABLED",
    "SDL_LOCALE_DUMMY",
    "SDL_MISC_DISABLED",
    "SDL_MISC_DUMMY",
    "SDL_POWER_ANDROID",
    "SDL_POWER_DISABLED",
    "SDL_POWER_EMSCRIPTEN",
    "SDL_POWER_HAIKU",
    "SDL_POWER_HARDWIRED",
    "SDL_POWER_LINUX",
    "SDL_POWER_MACOSX",
    "SDL_POWER_N3DS",
    "SDL_POWER_PSP",
    "SDL_POWER_UIKIT",
    "SDL_POWER_VITA",
    "SDL_POWER_WINDOWS",
    "SDL_POWER_WINRT",
    "SDL_RENDER_DISABLED",
    "SDL_SENSOR_ANDROID",
    "SDL_SENSOR_COREMOTION",
    "SDL_SENSOR_DISABLED",
    "SDL_SENSOR_DUMMY",
    "SDL_SENSOR_N3DS",
    "SDL_SENSOR_VITA",
    "SDL_SENSOR_WINDOWS",
    "SDL_THREADS_DISABLED",
    "SDL_THREAD_GENERIC_COND_SUFFIX",
    "SDL_THREAD_N3DS",
    "SDL_THREAD_OS2",
    "SDL_THREAD_PS2",
    "SDL_THREAD_PSP",
    "SDL_THREAD_PTHREAD",
    "SDL_THREAD_PTHREAD_RECURSIVE_MUTEX",
    "SDL_THREAD_PTHREAD_RECURSIVE_MUTEX_NP",
    "SDL_THREAD_VITA",
    "SDL_THREAD_WINDOWS",
    "SDL_TIMERS_DISABLED",
    "SDL_TIMER_DUMMY",
    "SDL_TIMER_HAIKU",
    "SDL_TIMER_N3DS",
    "SDL_TIMER_OS2",
    "SDL_TIMER_PS2",
    "SDL_TIMER_PSP",
    "SDL_TIMER_UNIX",
    "SDL_TIMER_VITA",
    "SDL_TIMER_WINDOWS",
    "SDL_UDEV_DYNAMIC",
    "SDL_USE_IME",
    "SDL_USE_LIBICONV",
    "SDL_VIDEO_DISABLED",
    "SDL_VIDEO_DRIVER_ANDROID",
    "SDL_VIDEO_DRIVER_COCOA",
    "SDL_VIDEO_DRIVER_DIRECTFB",
    "SDL_VIDEO_DRIVER_DIRECTFB_DYNAMIC",
    "SDL_VIDEO_DRIVER_DUMMY",
    "SDL_VIDEO_DRIVER_EMSCRIPTEN",
    "SDL_VIDEO_DRIVER_HAIKU",
    "SDL_VIDEO_DRIVER_KMSDRM",
    "SDL_VIDEO_DRIVER_KMSDRM_DYNAMIC",
    "SDL_VIDEO_DRIVER_KMSDRM_DYNAMIC_GBM",
    "SDL_VIDEO_DRIVER_N3DS",
    "SDL_VIDEO_DRIVER_OFFSCREEN",
    "SDL_VIDEO_DRIVER_OS2",
    "SDL_VIDEO_DRIVER_PS2",
    "SDL_VIDEO_DRIVER_PSP",
    "SDL_VIDEO_DRIVER_QNX",
    "SDL_VIDEO_DRIVER_RISCOS",
    "SDL_VIDEO_DRIVER_RPI",
    "SDL_VIDEO_DRIVER_UIKIT",
    "SDL_VIDEO_DRIVER_VITA",
    "SDL_VIDEO_DRIVER_VIVANTE",
    "SDL_VIDEO_DRIVER_VIVANTE_VDK",
    "SDL_VIDEO_DRIVER_WAYLAND",
    "SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC",
    "SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_CURSOR",
    "SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_EGL",
    "SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_LIBDECOR",
    "SDL_VIDEO_DRIVER_WAYLAND_DYNAMIC_XKBCOMMON",
    "SDL_VIDEO_DRIVER_WAYLAND_QT_TOUCH",
    "SDL_VIDEO_DRIVER_WINDOWS",
    "SDL_VIDEO_DRIVER_WINRT",
    "SDL_VIDEO_DRIVER_X11",
    "SDL_VIDEO_DRIVER_X11_DYNAMIC",
    "SDL_VIDEO_DRIVER_X11_DYNAMIC_XCURSOR",
    "SDL_VIDEO_DRIVER_X11_DYNAMIC_XEXT",
    "SDL_VIDEO_DRIVER_X11_DYNAMIC_XFIXES",
    "SDL_VIDEO_DRIVER_X11_DYNAMIC_XINPUT2",
    "SDL_VIDEO_DRIVER_X11_DYNAMIC_XRANDR",
    "SDL_VIDEO_DRIVER_X11_DYNAMIC_XSS",
    "SDL_VIDEO_DRIVER_X11_HAS_XKBKEYCODETOKEYSYM",
    "SDL_VIDEO_DRIVER_X11_SUPPORTS_GENERIC_EVENTS",
    "SDL_VIDEO_DRIVER_X11_XCURSOR",
    "SDL_VIDEO_DRIVER_X11_XDBE",
    "SDL_VIDEO_DRIVER_X11_XFIXES",
    "SDL_VIDEO_DRIVER_X11_XINPUT2",
    "SDL_VIDEO_DRIVER_X11_XINPUT2_SUPPORTS_MULTITOUCH",
    "SDL_VIDEO_DRIVER_X11_XRANDR",
    "SDL_VIDEO_DRIVER_X11_XSCRNSAVER",
    "SDL_VIDEO_DRIVER_X11_XSHAPE",
    "SDL_VIDEO_METAL",
    "SDL_VIDEO_OPENGL",
    "SDL_VIDEO_OPENGL_BGL",
    "SDL_VIDEO_OPENGL_CGL",
    "SDL_VIDEO_OPENGL_EGL",
    "SDL_VIDEO_OPENGL_ES",
    "SDL_VIDEO_OPENGL_ES2",
    "SDL_VIDEO_OPENGL_GLX",
    "SDL_VIDEO_OPENGL_OSMESA",
    "SDL_VIDEO_OPENGL_OSMESA_DYNAMIC",
    "SDL_VIDEO_OPENGL_WGL",
    "SDL_VIDEO_RENDER_D3D",
    "SDL_VIDEO_RENDER_D3D11",
    "SDL_VIDEO_RENDER_D3D12",
    "SDL_VIDEO_RENDER_DIRECTFB",
    "SDL_VIDEO_RENDER_METAL",
    "SDL_VIDEO_RENDER_OGL",
    "SDL_VIDEO_RENDER_OGL_ES",
    "SDL_VIDEO_RENDER_OGL_ES2",
    "SDL_VIDEO_RENDER_PS2",
    "SDL_VIDEO_RENDER_PSP",
    "SDL_VIDEO_RENDER_VITA_GXM",
    "SDL_VIDEO_VITA_PIB",
    "SDL_VIDEO_VITA_PVR",
    "SDL_VIDEO_VITA_PVR_OGL",
    "SDL_VIDEO_VULKAN",
}

if __name__ == "__main__":
    raise SystemExit(main())




#!/usr/bin/env python3
#
# This script renames SDL headers in the specified paths

import argparse
import pathlib
import re


def do_include_replacements(paths):
    replacements = [
        ( re.compile(r"(?:[\"<])(?:SDL2/)?SDL_image.h(?:[\">])"), r"<SDL3_image/SDL_image.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?SDL_mixer.h(?:[\">])"), r"<SDL3_mixer/SDL_mixer.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?SDL_net.h(?:[\">])"), r"<SDL3_net/SDL_net.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?SDL_rtf.h(?:[\">])"), r"<SDL3_rtf/SDL_rtf.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?SDL_ttf.h(?:[\">])"), r"<SDL3_ttf/SDL_ttf.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?SDL_gamecontroller.h(?:[\">])"), r"<SDL3/SDL_gamepad.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?begin_code.h(?:[\">])"), r"<SDL3/SDL_begin_code.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?close_code.h(?:[\">])"), r"<SDL3/SDL_close_code.h>" ),
        ( re.compile(r"(?:[\"<])(?:SDL2/)?(SDL[_a-z0-9]*\.h)(?:[\">])"), r"<SDL3/\1>" )
    ]
    for entry in paths:
        path = pathlib.Path(entry)
        if not path.exists():
            print("{} does not exist, skipping".format(entry))
            continue

        replace_headers_in_path(path, replacements)


def replace_headers_in_file(file, replacements):
    try:
        with file.open("r", encoding="UTF-8", newline="") as rfp:
            original = rfp.read()
            contents = original
            for regex, replacement in replacements:
                contents = regex.sub(replacement, contents)
            if contents != original:
                with file.open("w", encoding="UTF-8", newline="") as wfp:
                    wfp.write(contents)
    except UnicodeDecodeError:
        print("%s is not text, skipping" % file)
    except Exception as err:
        print("%s" % err)


def replace_headers_in_dir(path, replacements):
    for entry in path.glob("*"):
        if entry.is_dir():
            replace_headers_in_dir(entry, replacements)
        else:
            print("Processing %s" % entry)
            replace_headers_in_file(entry, replacements)


def replace_headers_in_path(path, replacements):
        if path.is_dir():
            replace_headers_in_dir(path, replacements)
        else:
            replace_headers_in_file(path, replacements)


def main():
    parser = argparse.ArgumentParser(fromfile_prefix_chars='@', description="Rename #include's for SDL3.")
    parser.add_argument("args", metavar="PATH", nargs="*", help="Input source file")
    args = parser.parse_args()

    try:
        do_include_replacements(args.args)
    except Exception as e:
        print(e)
        return 1

if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
import os
from argparse import ArgumentParser
from pathlib import Path
import re
import shutil
import sys
import textwrap


SDL_ROOT = Path(__file__).resolve().parents[1]

def extract_sdl_version() -> str:
    """
    Extract SDL version from SDL3/SDL_version.h
    """

    with open(SDL_ROOT / "include/SDL3/SDL_version.h") as f:
        data = f.read()

    major = int(next(re.finditer(r"#define\s+SDL_MAJOR_VERSION\s+([0-9]+)", data)).group(1))
    minor = int(next(re.finditer(r"#define\s+SDL_MINOR_VERSION\s+([0-9]+)", data)).group(1))
    micro = int(next(re.finditer(r"#define\s+SDL_MICRO_VERSION\s+([0-9]+)", data)).group(1))
    return f"{major}.{minor}.{micro}"

def replace_in_file(path: Path, regex_what: str, replace_with: str) -> None:
    with path.open("r") as f:
        data = f.read()

    new_data, count = re.subn(regex_what, replace_with, data)

    assert count > 0, f"\"{regex_what}\" did not match anything in \"{path}\""

    with open(path, "w") as f:
        f.write(new_data)


def android_mk_use_prefab(path: Path) -> None:
    """
    Replace relative SDL inclusion with dependency on prefab package
    """

    with path.open() as f:
        data = "".join(line for line in f.readlines() if "# SDL" not in line)

    data, _ = re.subn("[\n]{3,}", "\n\n", data)

    data, count = re.subn(r"(LOCAL_SHARED_LIBRARIES\s*:=\s*SDL3)", "LOCAL_SHARED_LIBRARIES := SDL3 SDL3-Headers", data)
    assert count == 1, f"Must have injected SDL3-Headers in {path} exactly once"

    newdata = data + textwrap.dedent("""
        # https://google.github.io/prefab/build-systems.html

        # Add the prefab modules to the import path.
        $(call import-add-path,/out)

        # Import SDL3 so we can depend on it.
        $(call import-module,prefab/SDL3)
    """)

    with path.open("w") as f:
        f.write(newdata)


def cmake_mk_no_sdl(path: Path) -> None:
    """
    Don't add the source directories of SDL/SDL_image/SDL_mixer/...
    """

    with path.open() as f:
        lines = f.readlines()

    newlines: list[str] = []
    for line in lines:
        if "add_subdirectory(SDL" in line:
            while newlines[-1].startswith("#"):
                newlines = newlines[:-1]
            continue
        newlines.append(line)

    newdata, _ = re.subn("[\n]{3,}", "\n\n", "".join(newlines))

    with path.open("w") as f:
        f.write(newdata)


def gradle_add_prefab_and_aar(path: Path, aar: str) -> None:
    with path.open() as f:
        data = f.read()

    data, count = re.subn("android {", textwrap.dedent("""
        android {
            buildFeatures {
                prefab true
            }"""), data)
    assert count == 1

    data, count = re.subn("dependencies {", textwrap.dedent(f"""
        dependencies {{
            implementation files('libs/{aar}')"""), data)
    assert count == 1

    with path.open("w") as f:
        f.write(data)


def gradle_add_package_name(path: Path, package_name: str) -> None:
    with path.open() as f:
        data = f.read()

    data, count = re.subn("org.libsdl.app", package_name, data)
    assert count >= 1

    with path.open("w") as f:
        f.write(data)


def main() -> int:
    description = "Create a simple Android gradle project from input sources."
    epilog = textwrap.dedent("""\
        You need to manually copy a prebuilt SDL3 Android archive into the project tree when using the aar variant.

        Any changes you have done to the sources in the Android project will be lost
    """)
    parser = ArgumentParser(description=description, epilog=epilog, allow_abbrev=False)
    parser.add_argument("package_name", metavar="PACKAGENAME", help="Android package name (e.g. com.yourcompany.yourapp)")
    parser.add_argument("sources", metavar="SOURCE", nargs="*", help="Source code of your application. The files are copied to the output directory.")
    parser.add_argument("--variant", choices=["copy", "symlink", "aar"], default="copy", help="Choose variant of SDL project (copy: copy SDL sources, symlink: symlink SDL sources, aar: use Android aar archive)")
    parser.add_argument("--output", "-o", default=SDL_ROOT / "build", type=Path, help="Location where to store the Android project")
    parser.add_argument("--version", default=None, help="SDL3 version to use as aar dependency (only used for aar variant)")

    args = parser.parse_args()
    if not args.sources:
        print("Reading source file paths from stdin (press CTRL+D to stop)")
        args.sources = [path for path in sys.stdin.read().strip().split() if path]
    if not args.sources:
        parser.error("No sources passed")

    if not os.getenv("ANDROID_HOME"):
        print("WARNING: ANDROID_HOME environment variable not set", file=sys.stderr)
    if not os.getenv("ANDROID_NDK_HOME"):
        print("WARNING: ANDROID_NDK_HOME environment variable not set", file=sys.stderr)

    args.sources = [Path(src) for src in args.sources]

    build_path = args.output / args.package_name

    # Remove the destination folder
    shutil.rmtree(build_path, ignore_errors=True)

    # Copy the Android project
    shutil.copytree(SDL_ROOT / "android-project", build_path)

    # Add the source files to the ndk-build and cmake projects
    replace_in_file(build_path / "app/jni/src/Android.mk", r"YourSourceHere\.c", " \\\n    ".join(src.name for src in args.sources))
    replace_in_file(build_path / "app/jni/src/CMakeLists.txt", r"YourSourceHere\.c", "\n    ".join(src.name for src in args.sources))

    # Remove placeholder source "YourSourceHere.c"
    (build_path / "app/jni/src/YourSourceHere.c").unlink()

    # Copy sources to output folder
    for src in args.sources:
        if not src.is_file():
            parser.error(f"\"{src}\" is not a file")
        shutil.copyfile(src, build_path / "app/jni/src" / src.name)

    sdl_project_files = (
        SDL_ROOT / "src",
        SDL_ROOT / "include",
        SDL_ROOT / "LICENSE.txt",
        SDL_ROOT / "README.md",
        SDL_ROOT / "Android.mk",
        SDL_ROOT / "CMakeLists.txt",
        SDL_ROOT / "cmake",
    )
    if args.variant == "copy":
        (build_path / "app/jni/SDL").mkdir(exist_ok=True, parents=True)
        for sdl_project_file in sdl_project_files:
            # Copy SDL project files and directories
            if sdl_project_file.is_dir():
                shutil.copytree(sdl_project_file, build_path / "app/jni/SDL" / sdl_project_file.name)
            elif sdl_project_file.is_file():
                shutil.copyfile(sdl_project_file, build_path / "app/jni/SDL" / sdl_project_file.name)
    elif args.variant == "symlink":
        (build_path / "app/jni/SDL").mkdir(exist_ok=True, parents=True)
        # Create symbolic links for all SDL project files
        for sdl_project_file in sdl_project_files:
            os.symlink(sdl_project_file, build_path / "app/jni/SDL" / sdl_project_file.name)
    elif args.variant == "aar":
        if not args.version:
            args.version = extract_sdl_version()

        major = args.version.split(".")[0]
        aar = f"SDL{ major }-{ args.version }.aar"

        # Remove all SDL java classes
        shutil.rmtree(build_path / "app/src/main/java")

        # Use prefab to generate include-able files
        gradle_add_prefab_and_aar(build_path / "app/build.gradle", aar=aar)

        # Make sure to use the prefab-generated files and not SDL sources
        android_mk_use_prefab(build_path / "app/jni/src/Android.mk")
        cmake_mk_no_sdl(build_path / "app/jni/CMakeLists.txt")

        aar_libs_folder = build_path / "app/libs"
        aar_libs_folder.mkdir(parents=True)
        with (aar_libs_folder / "copy-sdl-aars-here.txt").open("w") as f:
            f.write(f"Copy {aar} to this folder.\n")

        print(f"WARNING: copy { aar } to { aar_libs_folder }", file=sys.stderr)

    # Add the package name to build.gradle
    gradle_add_package_name(build_path / "app/build.gradle", args.package_name)

    # Create entry activity, subclassing SDLActivity
    activity = args.package_name[args.package_name.rfind(".") + 1:].capitalize() + "Activity"
    activity_path = build_path / "app/src/main/java" / args.package_name.replace(".", "/") / f"{activity}.java"
    activity_path.parent.mkdir(parents=True)
    with activity_path.open("w") as f:
        f.write(textwrap.dedent(f"""
            package {args.package_name};

            import org.libsdl.app.SDLActivity;

            public class {activity} extends SDLActivity
            {{
            }}
        """))

    # Add the just-generated activity to the Android manifest
    replace_in_file(build_path / "app/src/main/AndroidManifest.xml", 'name="SDLActivity"', f'name="{activity}"')

    # Update project and build
    print("To build and install to a device for testing, run the following:")
    print(f"cd {build_path}")
    print("./gradlew installDebug")
    return 0

if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
#
# This script renames symbols in the specified paths

import argparse
import os
import pathlib
import re
import sys


def main():
    if len(args.args) < 1:
        print("Usage: %s files_or_directories ..." % sys.argv[0])
        exit(1)

    replacements = {
        "SDL_bool": "bool",
        "SDL_TRUE": "true",
        "SDL_FALSE": "false",
    }
    entries = args.args[0:]

    regex = create_regex_from_replacements(replacements)

    for entry in entries:
        path = pathlib.Path(entry)
        if not path.exists():
            print("%s doesn't exist, skipping" % entry)
            continue

        replace_symbols_in_path(path, regex, replacements)

def create_regex_from_replacements(replacements):
    return re.compile(r"\b(%s)\b" % "|".join(map(re.escape, replacements.keys())))

def replace_symbols_in_file(file, regex, replacements):
    try:
        with file.open("r", encoding="UTF-8", newline="") as rfp:
            original = rfp.read()
            contents = regex.sub(lambda mo: replacements[mo.string[mo.start():mo.end()]], original)
            if contents != original:
                with file.open("w", encoding="UTF-8", newline="") as wfp:
                    wfp.write(contents)
    except UnicodeDecodeError:
        print("%s is not text, skipping" % file)
    except Exception as err:
        print("%s" % err)


def replace_symbols_in_dir(path, regex, replacements):
    for entry in path.glob("*"):
        if entry.is_dir():
            replace_symbols_in_dir(entry, regex, replacements)
        else:
            print("Processing %s" % entry)
            replace_symbols_in_file(entry, regex, replacements)


def replace_symbols_in_path(path, regex, replacements):
        if path.is_dir():
            replace_symbols_in_dir(path, regex, replacements)
        else:
            replace_symbols_in_file(path, regex, replacements)


if __name__ == "__main__":

    parser = argparse.ArgumentParser(fromfile_prefix_chars='@')
    parser.add_argument("args", nargs="*")
    args = parser.parse_args()

    try:
        main()
    except Exception as e:
        print(e)
        exit(-1)

    exit(0)




#!/usr/bin/env python

import argparse
import functools
import logging
import os
from pathlib import Path
import re
import shutil
import subprocess
import tempfile
import textwrap
import urllib.request
import zipfile

# Update both variables when updating the GDK
GIT_REF = "June_2024_Update_1"
GDK_EDITION = "240601"  # YYMMUU

logger = logging.getLogger(__name__)

class GdDesktopConfigurator:
    def __init__(self, gdk_path, arch, vs_folder, vs_version=None, vs_toolset=None, temp_folder=None, git_ref=None, gdk_edition=None):
        self.git_ref = git_ref or GIT_REF
        self.gdk_edition = gdk_edition or GDK_EDITION
        self.gdk_path = gdk_path
        self.temp_folder = temp_folder or Path(tempfile.gettempdir())
        self.dl_archive_path = Path(self.temp_folder) / f"{ self.git_ref }.zip"
        self.gdk_extract_path = Path(self.temp_folder) / f"GDK-{ self.git_ref }"
        self.arch = arch
        self.vs_folder = vs_folder
        self._vs_version = vs_version
        self._vs_toolset = vs_toolset

    def download_archive(self) -> None:
        gdk_url = f"https://github.com/microsoft/GDK/archive/refs/tags/{ GIT_REF }.zip"
        logger.info("Downloading %s to %s", gdk_url, self.dl_archive_path)
        urllib.request.urlretrieve(gdk_url, self.dl_archive_path)
        assert self.dl_archive_path.is_file()

    def extract_zip_archive(self) -> None:
        extract_path = self.gdk_extract_path.parent
        assert self.dl_archive_path.is_file()
        logger.info("Extracting %s to %s", self.dl_archive_path, extract_path)
        with zipfile.ZipFile(self.dl_archive_path) as zf:
            zf.extractall(extract_path)
        assert self.gdk_extract_path.is_dir(), f"{self.gdk_extract_path} must exist"

    def extract_development_kit(self) -> None:
        extract_dks_cmd = self.gdk_extract_path / "SetupScripts/ExtractXboxOneDKs.cmd"
        assert extract_dks_cmd.is_file()
        logger.info("Extracting GDK Development Kit: running %s", extract_dks_cmd)
        cmd = ["cmd.exe", "/C", str(extract_dks_cmd), str(self.gdk_extract_path), str(self.gdk_path)]
        logger.debug("Running %r", cmd)
        subprocess.check_call(cmd)

    def detect_vs_version(self) -> str:
        vs_regex = re.compile("VS([0-9]{4})")
        supported_vs_versions = []
        for p in self.gaming_grdk_build_path.iterdir():
            if not p.is_dir():
                continue
            if m := vs_regex.match(p.name):
                supported_vs_versions.append(m.group(1))
        logger.info(f"Supported Visual Studio versions: {supported_vs_versions}")
        vs_versions = set(self.vs_folder.parts).intersection(set(supported_vs_versions))
        if not vs_versions:
            raise RuntimeError("Visual Studio version is incompatible")
        if len(vs_versions) > 1:
            raise RuntimeError(f"Too many compatible VS versions found ({vs_versions})")
        vs_version = vs_versions.pop()
        logger.info(f"Used Visual Studio version: {vs_version}")
        return vs_version

    def detect_vs_toolset(self) -> str:
        toolset_paths = []
        for ts_path in self.gdk_toolset_parent_path.iterdir():
            if not ts_path.is_dir():
                continue
            ms_props = ts_path / "Microsoft.Cpp.props"
            if not ms_props.is_file():
                continue
            toolset_paths.append(ts_path.name)
        logger.info("Detected Visual Studio toolsets: %s", toolset_paths)
        assert toolset_paths, "Have we detected at least one toolset?"

        def toolset_number(toolset: str) -> int:
            if m:= re.match("[^0-9]*([0-9]+).*", toolset):
                return int(m.group(1))
            return -9

        return max(toolset_paths, key=toolset_number)

    @property
    def vs_version(self) -> str:
        if self._vs_version is None:
            self._vs_version = self.detect_vs_version()
        return self._vs_version

    @property
    def vs_toolset(self) -> str:
        if self._vs_toolset is None:
            self._vs_toolset = self.detect_vs_toolset()
        return self._vs_toolset

    @staticmethod
    def copy_files_and_merge_into(srcdir: Path, dstdir: Path) -> None:
        logger.info(f"Copy {srcdir} to {dstdir}")
        for root, _, files in os.walk(srcdir):
            dest_root = dstdir / Path(root).relative_to(srcdir)
            if not dest_root.is_dir():
                dest_root.mkdir()
            for file in files:
                srcfile = Path(root) / file
                dstfile = dest_root / file
                shutil.copy(srcfile, dstfile)

    def copy_msbuild(self) -> None:
        vc_toolset_parent_path = self.vs_folder / "MSBuild/Microsoft/VC"
        if 1:
            logger.info(f"Detected compatible Visual Studio version: {self.vs_version}")
            srcdir = vc_toolset_parent_path
            dstdir = self.gdk_toolset_parent_path
            assert srcdir.is_dir(), "Source directory must exist"
            assert dstdir.is_dir(), "Destination directory must exist"

            self.copy_files_and_merge_into(srcdir=srcdir, dstdir=dstdir)

    @property
    def game_dk_path(self) -> Path:
        return self.gdk_path / "Microsoft GDK"

    @property
    def game_dk_latest_path(self) -> Path:
        return self.game_dk_path / self.gdk_edition

    @property
    def windows_sdk_path(self) -> Path:
        return self.gdk_path / "Windows Kits/10"

    @property
    def gaming_grdk_build_path(self) -> Path:
        return self.game_dk_latest_path / "GRDK"

    @property
    def gdk_toolset_parent_path(self) -> Path:
        return self.gaming_grdk_build_path / f"VS{self.vs_version}/flatDeployment/MSBuild/Microsoft/VC"

    @property
    def env(self) -> dict[str, str]:
        game_dk = self.game_dk_path
        game_dk_latest = self.game_dk_latest_path
        windows_sdk_dir = self.windows_sdk_path
        gaming_grdk_build = self.gaming_grdk_build_path

        return {
            "GRDKEDITION": f"{self.gdk_edition}",
            "GameDK": f"{game_dk}\\",
            "GameDKLatest": f"{ game_dk_latest }\\",
            "WindowsSdkDir": f"{ windows_sdk_dir }\\",
            "GamingGRDKBuild": f"{ gaming_grdk_build }\\",
            "VSInstallDir": f"{ self.vs_folder }\\",
        }

    def create_user_props(self, path: Path) -> None:
        vc_targets_path = self.gaming_grdk_build_path / f"VS{ self.vs_version }/flatDeployment/MSBuild/Microsoft/VC/{ self.vs_toolset }"
        vc_targets_path16 = self.gaming_grdk_build_path / f"VS2019/flatDeployment/MSBuild/Microsoft/VC/{ self.vs_toolset }"
        vc_targets_path17 = self.gaming_grdk_build_path / f"VS2022/flatDeployment/MSBuild/Microsoft/VC/{ self.vs_toolset }"
        additional_include_directories = ";".join(str(p) for p in self.gdk_include_paths)
        additional_library_directories = ";".join(str(p) for p in self.gdk_library_paths)
        durango_xdk_install_path = self.gdk_path / "Microsoft GDK"
        with path.open("w") as f:
            f.write(textwrap.dedent(f"""\
                <?xml version="1.0" encoding="utf-8"?>
                <Project ToolsVersion="4.0" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
                  <PropertyGroup>
                    <VCTargetsPath>{ vc_targets_path }\\</VCTargetsPath>
                    <VCTargetsPath16>{ vc_targets_path16 }\\</VCTargetsPath16>
                    <VCTargetsPath17>{ vc_targets_path17 }\\</VCTargetsPath17>
                    <BWOI_GDK_Path>{ self.gaming_grdk_build_path }\\</BWOI_GDK_Path>
                    <Platform Condition="'$(Platform)' == ''">Gaming.Desktop.x64</Platform>
                    <Configuration Condition="'$(Configuration)' == ''">Debug</Configuration>
                    <XdkEditionTarget>{ self.gdk_edition }</XdkEditionTarget>
                    <DurangoXdkInstallPath>{ durango_xdk_install_path }</DurangoXdkInstallPath>

                    <DefaultXdkEditionRootVS2019>$(DurangoXdkInstallPath)\\{self.gdk_edition}\\GRDK\\VS2019\\flatDeployment\\MSBuild\\Microsoft\\VC\\{self.vs_toolset}\\Platforms\\$(Platform)\\</DefaultXdkEditionRootVS2019>
                    <XdkEditionRootVS2019>$(DurangoXdkInstallPath)\\{self.gdk_edition}\\GRDK\\VS2019\\flatDeployment\\MSBuild\\Microsoft\\VC\\{self.vs_toolset}\\Platforms\\$(Platform)\\</XdkEditionRootVS2019>
                    <DefaultXdkEditionRootVS2022>$(DurangoXdkInstallPath)\\{self.gdk_edition}\\GRDK\\VS2022\\flatDeployment\\MSBuild\\Microsoft\\VC\\{self.vs_toolset}\\Platforms\\$(Platform)\\</DefaultXdkEditionRootVS2022>
                    <XdkEditionRootVS2022>$(DurangoXdkInstallPath)\\{self.gdk_edition}\\GRDK\\VS2022\\flatDeployment\\MSBuild\\Microsoft\\VC\\{self.vs_toolset}\\Platforms\\$(Platform)\\</XdkEditionRootVS2022>

                    <Deterministic>true</Deterministic>
                    <DisableInstalledVCTargetsUse>true</DisableInstalledVCTargetsUse>
                    <ClearDevCommandPromptEnvVars>true</ClearDevCommandPromptEnvVars>
                  </PropertyGroup>
                  <ItemDefinitionGroup Condition="'$(Platform)' == 'Gaming.Desktop.x64'">
                    <ClCompile>
                      <AdditionalIncludeDirectories>{ additional_include_directories };%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>
                    </ClCompile>
                    <Link>
                      <AdditionalLibraryDirectories>{ additional_library_directories };%(AdditionalLibraryDirectories)</AdditionalLibraryDirectories>
                    </Link>
                  </ItemDefinitionGroup>
                </Project>
            """))

    @property
    def gdk_include_paths(self) -> list[Path]:
        return [
            self.gaming_grdk_build_path / "gamekit/include",
        ]

    @property
    def gdk_library_paths(self) -> list[Path]:
        return [
            self.gaming_grdk_build_path / f"gamekit/lib/{self.arch}",
        ]

    @property
    def gdk_binary_path(self) -> list[Path]:
        return [
            self.gaming_grdk_build_path / "bin",
            self.game_dk_path / "bin",
        ]

    @property
    def build_env(self) -> dict[str, str]:
        gdk_include = ";".join(str(p) for p in self.gdk_include_paths)
        gdk_lib = ";".join(str(p) for p in self.gdk_library_paths)
        gdk_path = ";".join(str(p) for p in self.gdk_binary_path)
        return {
            "GDK_INCLUDE": gdk_include,
            "GDK_LIB": gdk_lib,
            "GDK_PATH": gdk_path,
        }

    def print_env(self) -> None:
        for k, v in self.env.items():
            print(f"set \"{k}={v}\"")
        print()
        for k, v in self.build_env.items():
            print(f"set \"{k}={v}\"")
        print()
        print(f"set \"PATH=%GDK_PATH%;%PATH%\"")
        print(f"set \"LIB=%GDK_LIB%;%LIB%\"")
        print(f"set \"INCLUDE=%GDK_INCLUDE%;%INCLUDE%\"")


def main():
    logging.basicConfig(level=logging.INFO)
    parser = argparse.ArgumentParser(allow_abbrev=False)
    parser.add_argument("--arch", choices=["amd64"], default="amd64", help="Architecture")
    parser.add_argument("--download", action="store_true", help="Download GDK")
    parser.add_argument("--extract", action="store_true", help="Extract downloaded GDK")
    parser.add_argument("--copy-msbuild", action="store_true", help="Copy MSBuild files")
    parser.add_argument("--temp-folder", help="Temporary folder where to download and extract GDK")
    parser.add_argument("--gdk-path", required=True, type=Path, help="Folder where to store the GDK")
    parser.add_argument("--ref-edition", type=str, help="Git ref and GDK edition separated by comma")
    parser.add_argument("--vs-folder", required=True, type=Path, help="Installation folder of Visual Studio")
    parser.add_argument("--vs-version", required=False, type=int, help="Visual Studio version")
    parser.add_argument("--vs-toolset", required=False, help="Visual Studio toolset (e.g. v150)")
    parser.add_argument("--props-folder", required=False, type=Path, default=Path(), help="Visual Studio toolset (e.g. v150)")
    parser.add_argument("--no-user-props", required=False, dest="user_props", action="store_false", help="Don't ")
    args = parser.parse_args()

    logging.basicConfig(level=logging.INFO)

    git_ref = None
    gdk_edition = None
    if args.ref_edition is not None:
        git_ref, gdk_edition = args.ref_edition.split(",", 1)
        try:
            int(gdk_edition)
        except ValueError:
            parser.error("Edition should be an integer (YYMMUU) (Y=year M=month U=update)")

    configurator = GdDesktopConfigurator(
        arch=args.arch,
        git_ref=git_ref,
        gdk_edition=gdk_edition,
        vs_folder=args.vs_folder,
        vs_version=args.vs_version,
        vs_toolset=args.vs_toolset,
        gdk_path=args.gdk_path,
        temp_folder=args.temp_folder,
    )

    if args.download:
        configurator.download_archive()

    if args.extract:
        configurator.extract_zip_archive()

        configurator.extract_development_kit()

    if args.copy_msbuild:
        configurator.copy_msbuild()

    if args.user_props:
        configurator.print_env()
        configurator.create_user_props(args.props_folder / "Directory.Build.props")

if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python

import argparse
import dataclasses
import os
import pathlib
import re

ROOT = pathlib.Path(__file__).resolve().parents[1]
SDL_ANDROID_C = ROOT / "src/core/android/SDL_android.c"
METHOD_SOURCE_PATHS = (
    SDL_ANDROID_C,
    ROOT / "src/hidapi/android/hid.cpp",
)
JAVA_ROOT = ROOT / "android-project/app/src/main/java"


BASIC_TYPE_SPEC_LUT = {
    "char": "C",
    "byte": "B",
    "short": "S",
    "int": "I",
    "long": "J",
    "float": "F",
    "double": "D",
    "void": "V",
    "boolean": "Z",
    "Object": "Ljava/lang/Object;",
    "String": "Ljava/lang/String;",
}


@dataclasses.dataclass(frozen=True)
class JniType:
    typ: str
    array: int


def java_type_to_jni_spec_internal(type_str: str) -> tuple[int, str]:
    for basic_type_str, basic_type_spec in BASIC_TYPE_SPEC_LUT.items():
        if type_str.startswith(basic_type_str):
            return len(basic_type_str), basic_type_spec
    raise ValueError(f"Don't know how to convert {repr(type_str)} to its equivalent jni spec")


def java_type_to_jni_spec(type_str: str) -> str:
    end, type_spec = java_type_to_jni_spec_internal(type_str)
    suffix_str = type_str[end:]
    assert(all(c in "[] \t" for c in suffix_str))
    suffix_str = "".join(filter(lambda v: v in "[]", suffix_str))
    assert len(suffix_str) % 2 == 0
    array_spec = "[" * (len(suffix_str) // 2)
    return array_spec + type_spec


def java_method_to_jni_spec(ret: str, args: list[str]) -> str:
    return "(" + "".join(java_type_to_jni_spec(a) for a in args) +")" + java_type_to_jni_spec(ret)


@dataclasses.dataclass(frozen=True)
class JniMethodBinding:
    name: str
    spec: str


def collect_jni_bindings_from_c() -> dict[str, set[JniMethodBinding]]:
    bindings = {}

    sdl_android_text = SDL_ANDROID_C.read_text()
    for m in re.finditer(r"""register_methods\((?:[A-Za-z0-9]+),\s*"(?P<class>[a-zA-Z0-9_/]+)",\s*(?P<table>[a-zA-Z0-9_]+),\s*SDL_arraysize\((?P=table)\)\)""", sdl_android_text):
        kls = m["class"]
        table = m["table"]
        methods = set()
        in_struct = False
        for method_source_path in METHOD_SOURCE_PATHS:
            method_source = method_source_path.read_text()
            for line in method_source.splitlines(keepends=False):
                if re.match(f"(static )?JNINativeMethod {table}" + r"\[([0-9]+)?\] = \{", line):
                    in_struct = True
                    continue
                if in_struct:
                    if re.match(r"\};", line):
                        in_struct = False
                        break
                    n = re.match(r"""\s*\{\s*"(?P<method>[a-zA-Z0-9_]+)"\s*,\s*"(?P<spec>[()A-Za-z0-9_/;[]+)"\s*,\s*(\(void\*\))?(HID|SDL)[_A-Z]*_JAVA_[_A-Z]*INTERFACE[_A-Z]*\((?P=method)\)\s*\},?""", line)
                    assert n, f"'{line}' does not match regex"
                    methods.add(JniMethodBinding(name=n["method"], spec=n["spec"]))
                    continue
                if methods:
                    break
            if methods:
                break
        assert methods, f"Could not find methods for {kls} (table={table})"

        assert not in_struct

        assert kls not in bindings, f"{kls} must be unique in C sources"
        bindings[kls] = methods
    return bindings

def collect_jni_bindings_from_java() -> dict[str, set[JniMethodBinding]]:
    bindings = {}

    for root, _, files in os.walk(JAVA_ROOT):
        for file in files:
            file_path = pathlib.Path(root) / file
            java_text = file_path.read_text()
            methods = set()
            for m in re.finditer(r"(?:(?:public|private)\s+)?(?:static\s+)?native\s+(?P<ret>[A-Za-z0-9_]+)\s+(?P<method>[a-zA-Z0-9_]+)\s*\(\s*(?P<args>[^)]*)\);", java_text):
                name = m["method"]
                ret = m["ret"]
                args = []
                args_str = m["args"].strip()
                if args_str:
                    for a_s in args_str.split(","):
                        atype_str, _ = a_s.strip().rsplit(" ")
                        args.append(atype_str.strip())

                spec = java_method_to_jni_spec(ret=ret, args=args)
                methods.add(JniMethodBinding(name=name, spec=spec))
            if methods:
                relative_java_path = file_path.relative_to(JAVA_ROOT)
                relative_java_path_without_suffix = relative_java_path.with_suffix("")
                kls = "/".join(relative_java_path_without_suffix.parts)
                assert kls not in bindings, f"{kls} must be unique in JAVA sources"
                bindings[kls] = methods
    return bindings


def print_error(*args):
    print("ERROR:", *args)


def main():
    parser = argparse.ArgumentParser(allow_abbrev=False, description="Verify Android JNI bindings")
    args = parser.parse_args()

    bindings_from_c = collect_jni_bindings_from_c()
    bindings_from_java = collect_jni_bindings_from_java()

    all_ok = bindings_from_c == bindings_from_java
    if all_ok:
        print("OK")
    else:
        print("NOT OK")
        kls_c = set(bindings_from_c.keys())
        kls_java = set(bindings_from_java.keys())
        if kls_c != kls_java:
            only_c = kls_c - kls_java
            for c in only_c:
                print_error(f"Missing class in JAVA sources: {c}")
            only_java = kls_java - kls_c
            for c in only_java:
                print_error(f"Missing class in C sources: {c}")

        klasses = kls_c.union(kls_java)
        for kls in klasses:
            m_c = bindings_from_c.get(kls)
            m_j = bindings_from_java.get(kls)
            if m_c and m_j and m_c != m_j:
                m_only_c = m_c - m_j
                for c in m_only_c:
                    print_error(f"{kls}: Binding only in C source: {c.name} {c.spec}")
                m_only_j = m_j - m_c
                for c in m_only_j:
                    print_error(f"{kls}: Binding only in JAVA source: {c.name} {c.spec}")

    return 0 if all_ok else 1


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python

# Based on http/server.py from Python

from argparse import ArgumentParser
import contextlib
from http.server import SimpleHTTPRequestHandler
from http.server import ThreadingHTTPServer
import os
import socket


class MyHTTPRequestHandler(SimpleHTTPRequestHandler):
    extensions_map = {
        ".manifest": "text/cache-manifest",
        ".html": "text/html",
        ".png": "image/png",
        ".jpg": "image/jpg",
        ".svg":	"image/svg+xml",
        ".css":	"text/css",
        ".js":	"application/x-javascript",
        ".wasm": "application/wasm",
        "": "application/octet-stream",
    }

    def __init__(self, *args, maps=None, **kwargs):
        self.maps = maps or []
        SimpleHTTPRequestHandler.__init__(self, *args, **kwargs)

    def end_headers(self):
        self.send_my_headers()
        SimpleHTTPRequestHandler.end_headers(self)

    def send_my_headers(self):
        self.send_header("Cache-Control", "no-cache, no-store, must-revalidate")
        self.send_header("Pragma", "no-cache")
        self.send_header("Expires", "0")

    def translate_path(self, path):
        for map_path, map_prefix in self.maps:
            if path.startswith(map_prefix):
                res = os.path.join(map_path, path.removeprefix(map_prefix).lstrip("/"))
                break
        else:
            res = super().translate_path(path)
        return res


def serve_forever(port: int, ServerClass):
    handler = MyHTTPRequestHandler

    addr = ("0.0.0.0", port)
    with ServerClass(addr, handler) as httpd:
        host, port = httpd.socket.getsockname()[:2]
        url_host = f"[{host}]" if ":" in host else host
        print(f"Serving HTTP on {host} port {port} (http://{url_host}:{port}/) ...")
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            print("\nKeyboard interrupt received, exiting.")
            return 0


def main():
    parser = ArgumentParser(allow_abbrev=False)
    parser.add_argument("port", nargs="?", type=int, default=8080)
    parser.add_argument("-d", dest="directory", type=str, default=None)
    parser.add_argument("--map", dest="maps", nargs="+", type=str, help="Mappings, used as e.g. \"$HOME/projects/SDL:/sdl\"")
    args = parser.parse_args()

    maps = []
    for m in args.maps:
        try:
            path, uri  = m.split(":", 1)
        except ValueError:
            parser.error(f"Invalid mapping: \"{m}\"")
        maps.append((path, uri))

    class DualStackServer(ThreadingHTTPServer):
        def server_bind(self):
            # suppress exception when protocol is IPv4
            with contextlib.suppress(Exception):
                self.socket.setsockopt(socket.IPPROTO_IPV6, socket.IPV6_V6ONLY, 0)
            return super().server_bind()

        def finish_request(self, request, client_address):
            self.RequestHandlerClass(
                request,
                client_address,
                self,
                directory=args.directory,
                maps=maps,
            )

    return serve_forever(
        port=args.port,
        ServerClass=DualStackServer,
    )


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
#
#  Simple DirectMedia Layer
#  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
#
#  This software is provided 'as-is', without any express or implied
#  warranty.  In no event will the authors be held liable for any damages
#  arising from the use of this software.
#
#  Permission is granted to anyone to use this software for any purpose,
#  including commercial applications, and to alter it and redistribute it
#  freely, subject to the following restrictions:
#
#  1. The origin of this software must not be misrepresented; you must not
#     claim that you wrote the original software. If you use this software
#     in a product, an acknowledgment in the product documentation would be
#     appreciated but is not required.
#  2. Altered source versions must be plainly marked as such, and must not be
#     misrepresented as being the original software.
#  3. This notice may not be removed or altered from any source distribution.
#
# This script detects use of stdlib function in SDL code

import argparse
import os
import pathlib
import re
import sys

SDL_ROOT = pathlib.Path(__file__).resolve().parents[1]

STDLIB_SYMBOLS = [
    'abs',
    'acos',
    'acosf',
    'asin',
    'asinf',
    'asprintf',
    'atan',
    'atan2',
    'atan2f',
    'atanf',
    'atof',
    'atoi',
    'bsearch',
    'calloc',
    'ceil',
    'ceilf',
    'copysign',
    'copysignf',
    'cos',
    'cosf',
    'crc32',
    'exp',
    'expf',
    'fabs',
    'fabsf',
    'floor',
    'floorf',
    'fmod',
    'fmodf',
    'free',
    'getenv',
    'isalnum',
    'isalpha',
    'isblank',
    'iscntrl',
    'isdigit',
    'isgraph',
    'islower',
    'isprint',
    'ispunct',
    'isspace',
    'isupper',
    'isxdigit',
    'itoa',
    'lltoa',
    'log10',
    'log10f',
    'logf',
    'lround',
    'lroundf',
    'ltoa',
    'malloc',
    'memalign',
    'memcmp',
    'memcpy',
    'memcpy4',
    'memmove',
    'memset',
    'pow',
    'powf',
    'qsort',
    'qsort_r',
    'qsort_s',
    'realloc',
    'round',
    'roundf',
    'scalbn',
    'scalbnf',
    'setenv',
    'sin',
    'sinf',
    'snprintf',
    'sqrt',
    'sqrtf',
    'sscanf',
    'strcasecmp',
    'strchr',
    'strcmp',
    'strdup',
    'strlcat',
    'strlcpy',
    'strlen',
    'strlwr',
    'strncasecmp',
    'strncmp',
    'strrchr',
    'strrev',
    'strstr',
    'strtod',
    'strtokr',
    'strtol',
    'strtoll',
    'strtoul',
    'strupr',
    'tan',
    'tanf',
    'tolower',
    'toupper',
    'trunc',
    'truncf',
    'uitoa',
    'ulltoa',
    'ultoa',
    'utf8strlcpy',
    'utf8strlen',
    'vasprintf',
    'vsnprintf',
    'vsscanf',
    'wcscasecmp',
    'wcscmp',
    'wcsdup',
    'wcslcat',
    'wcslcpy',
    'wcslen',
    'wcsncasecmp',
    'wcsncmp',
    'wcsstr',
]
RE_STDLIB_SYMBOL = re.compile(rf"\b(?P<symbol>{'|'.join(STDLIB_SYMBOLS)})\b\(")


def find_symbols_in_file(file: pathlib.Path) -> int:
    match_count = 0

    allowed_extensions = [ ".c", ".cpp", ".m", ".h",  ".hpp", ".cc" ]

    excluded_paths = [
        "src/stdlib",
        "src/libm",
        "src/hidapi",
        "src/video/khronos",
        "src/video/stb_image.h",
        "include/SDL3",
        "build-scripts/gen_audio_resampler_filter.c",
        "build-scripts/gen_audio_channel_conversion.c",
        "test/win32/sdlprocdump.c",
    ]

    filename = pathlib.Path(file)

    for ep in excluded_paths:
        if ep in filename.as_posix():
            # skip
            return 0

    if filename.suffix not in allowed_extensions:
        # skip
        return 0

    # print("Parse %s" % file)

    try:
        with file.open("r", encoding="UTF-8", newline="") as rfp:
            parsing_comment = False
            for line_i, original_line in enumerate(rfp, start=1):
                line = original_line.strip()

                line_comment = ""

                # Get the comment block /* ... */ across several lines
                while True:
                    if parsing_comment:
                        pos_end_comment = line.find("*/")
                        if pos_end_comment >= 0:
                            line = line[pos_end_comment+2:]
                            parsing_comment = False
                        else:
                            break
                    else:
                        pos_start_comment = line.find("/*")
                        if pos_start_comment >= 0:
                            pos_end_comment = line.find("*/", pos_start_comment+2)
                            if pos_end_comment >= 0:
                                line_comment += line[pos_start_comment:pos_end_comment+2]
                                line = line[:pos_start_comment] + line[pos_end_comment+2:]
                            else:
                                line_comment += line[pos_start_comment:]
                                line = line[:pos_start_comment]
                                parsing_comment = True
                                break
                        else:
                            break
                if parsing_comment:
                    continue
                pos_line_comment = line.find("//")
                if pos_line_comment >= 0:
                    line_comment += line[pos_line_comment:]
                    line = line[:pos_line_comment]

                if m := RE_STDLIB_SYMBOL.match(line):
                    override_string = f"This should NOT be SDL_{m['symbol']}()"
                    if override_string not in line_comment:
                        print(f"{filename}:{line_i}")
                        print(f"    {line}")
                        print(f"")
                        match_count += 1

    except UnicodeDecodeError:
        print(f"{file} is not text, skipping", file=sys.stderr)

    return match_count

def find_symbols_in_dir(path: pathlib.Path) -> int:
    match_count = 0
    for entry in path.glob("*"):
        if entry.is_dir():
            match_count += find_symbols_in_dir(entry)
        else:
            match_count += find_symbols_in_file(entry)
    return match_count

def main():
    parser = argparse.ArgumentParser(fromfile_prefix_chars="@")
    parser.add_argument("path", default=SDL_ROOT, nargs="?", type=pathlib.Path, help="Path to look for stdlib symbols")
    args = parser.parse_args()

    print(f"Looking for stdlib usage in {args.path}...")

    match_count = find_symbols_in_dir(args.path)

    if match_count:
        print("If the stdlib usage is intentional, add a '// This should NOT be SDL_<symbol>()' line comment.")
        print("")
        print("NOT OK")
    else:
        print("OK")
    return 1 if match_count else 0

if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python

import argparse
import os
import pathlib
import re

def main():
    parser = argparse.ArgumentParser(allow_abbrev=False, description="Convert file into includable C header")
    parser.add_argument("--in", "-i", type=pathlib.Path, metavar="INPUT", dest="input", required=True, help="Input file")
    parser.add_argument("--out", "-o", type=pathlib.Path, metavar="OUTPUT", dest="output", required=True, help="Output header")
    parser.add_argument("--columns", type=int, default=12, help="Column count")
    args = parser.parse_args()

    t = pathlib.Path()
    varname, _ = re.subn("[^a-zA-Z0-9]", "_", str(args.input.name))

    binary_data = args.input.open("rb").read()

    with args.output.open("w", newline="\n") as fout:
        fout.write("unsigned char {}[] = {{\n".format(varname))
        bytes_written = 0
        while bytes_written < len(binary_data):
            col = bytes_written % args.columns
            if col == 0:
                fout.write("  ")
            column_data = binary_data[bytes_written:bytes_written+args.columns]
            fout.write(", ".join("0x{:02x}".format(d) for d in column_data))
            bytes_written += len(column_data)
            if bytes_written < len(binary_data):
                fout.write(",\n")
            else:
                fout.write("\n")
        fout.write("}};\nunsigned int {}_len = {:d};\n".format(varname, len(binary_data)))

if __name__ == "__main__":
    raise SystemExit(main())
